@@ -1,18 +1,19 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2001-2013. All Rights Reserved.
+%% Copyright Ericsson AB 2001-2017. All Rights Reserved.
 %%
-%% The contents of this file are subject to the Erlang Public License,
-%% Version 1.1, (the "License"); you may not use this file except in
-%% compliance with the License. You should have received a copy of the
-%% Erlang Public License along with this software. If not, it can be
-%% retrieved online at http://www.erlang.org/.
+%% Licensed under the Apache License, Version 2.0 (the "License");
+%% you may not use this file except in compliance with the License.
+%% You may obtain a copy of the License at
 %%
-%% Software distributed under the License is distributed on an "AS IS"
-%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
-%% the License for the specific language governing rights and limitations
-%% under the License.
+%%     http://www.apache.org/licenses/LICENSE-2.0
+%%
+%% Unless required by applicable law or agreed to in writing, software
+%% distributed under the License is distributed on an "AS IS" BASIS,
+%% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+%% See the License for the specific language governing permissions and
+%% limitations under the License.
 %%
 %% %CopyrightEnd%
 %%
@@ -20,6 +21,9 @@
 
 -module(asn1_SUITE).
 
+%% Suppress compilation of an addititional module compiled for maps.
+-define(NO_MAPS_MODULE, asn1_test_lib_no_maps).
+
 -define(only_ber(Func),
     if Rule =:= ber -> Func;
        true -> ok
@@ -27,19 +31,22 @@
 
 -compile(export_all).
 
--include_lib("test_server/include/test_server.hrl").
+-include_lib("common_test/include/ct.hrl").
 
 %%------------------------------------------------------------------------------
 %% Suite definition
 %%------------------------------------------------------------------------------
 
-suite() -> [{ct_hooks, [ts_install_cth]}].
+suite() ->
+    [{ct_hooks, [ts_install_cth]},
+     {timetrap,{minutes,60}}].
 
 all() ->
-    [{group, compile},
+    [xref,
+     xref_export_all,
+
+     {group, compile},
      {group, parallel},
-     {group, app_test},
-     {group, appup_test},
 
      % TODO: Investigate parallel running of these:
      testComment,
@@ -52,9 +59,7 @@ all() ->
 groups() ->
     Parallel = asn1_test_lib:parallel(),
     [{compile, Parallel,
-      [c_syntax,
-       c_string,
-       c_implicit_before_choice,
+      [c_string,
        constraint_equivalence]},
 
      {ber, Parallel,
@@ -63,13 +68,8 @@ groups() ->
        ber_optional,
        tagdefault_automatic]},
 
-     {app_test, [], [{asn1_app_test, all}]},
-
-     {appup_test, [], [{asn1_appup_test, all}]},
-
      {parallel, Parallel,
       [cover,
-       xref,
        {group, ber},
        % Uses 'P-Record', 'Constraints', 'MEDIA-GATEWAY-CONTROL'...
        {group, [], [parse,
@@ -89,6 +89,7 @@ groups() ->
                     ber_other,
 		    der,
                     h323test]},
+       testExtensibilityImplied,
        testChoPrim,
        testChoExtension,
        testChoOptional,
@@ -100,6 +101,7 @@ groups() ->
        testMultipleLevels,
        testOpt,
        testSeqDefault,
+       testMaps,
        % Uses 'External'
        {group, [], [testExternal,
                     testSeqExtension]},
@@ -135,19 +137,20 @@ groups() ->
        testChoiceIndefinite,
        per_open_type,
        testInfObjectClass,
+       testUniqueObjectSets,
+       testInfObjExtract,
        testParam,
        testFragmented,
        testMergeCompile,
        testobj,
        testDeepTConstr,
-       testExport,
        testImport,
        testDER,
        testDEFAULT,
+       testExtensionDefault,
        testMvrasn6,
        testContextSwitchingTypes,
        testOpenTypeImplicitTag,
-       duplicate_tags,
        testROSE,
        testINSTANCE_OF,
        testTCAP,
@@ -158,21 +161,27 @@ groups() ->
        testNortel,
        % Uses 'PKCS7', 'InformationFramework'
        {group, [], [test_WS_ParamClass,
-		    test_modified_x420,
-                    testX420]},
-       testTcapsystem,
-       testNBAPsystem,
-       testS1AP,
+		    test_modified_x420]},
+       %% Don't run all these at the same time.
+       {group, [],
+	[testTcapsystem,
+	 testNBAPsystem,
+	 testS1AP,
+	 testRfcs]},
        test_compile_options,
        testDoubleEllipses,
        test_x691,
        ticket_6143,
-       test_OTP_9688]},
+       test_OTP_9688,
+       testValueTest]},
 
      {performance, [],
       [testTimer_ber,
+       testTimer_ber_maps,
        testTimer_per,
-       testTimer_uper]}].
+       testTimer_per_maps,
+       testTimer_uper,
+       testTimer_uper_maps]}].
 
 %%------------------------------------------------------------------------------
 %% Init/end
@@ -191,18 +200,15 @@ end_per_group(_GroupName, Config) ->
     Config.
 
 init_per_testcase(Func, Config) ->
-    CaseDir = filename:join(?config(priv_dir, Config), Func),
+    CaseDir = filename:join(proplists:get_value(priv_dir, Config), Func),
     ok = filelib:ensure_dir(filename:join([CaseDir, dummy_file])),
     true = code:add_patha(CaseDir),
-
-    Dog = case Func of
-              testX420 -> ct:timetrap({minutes, 90});
-              _        -> ct:timetrap({minutes, 60})
-          end,
-    [{case_dir, CaseDir}, {watchdog, Dog}|Config].
+    [{case_dir, CaseDir}|Config].
 
 end_per_testcase(_Func, Config) ->
-    code:del_path(?config(case_dir, Config)).
+    CaseDir = proplists:get_value(case_dir, Config),
+    asn1_test_lib:rm_dirs([CaseDir]),
+    code:del_path(CaseDir).
 
 %%------------------------------------------------------------------------------
 %% Test runners
@@ -240,7 +246,7 @@ opts(Rule) when is_atom(Rule) -> [];
 opts({_Rule, Opts})         -> Opts.
 
 run_case(Config, Fun, Rule, Opts) ->
-    CaseDir = ?config(case_dir, Config),
+    CaseDir = proplists:get_value(case_dir, Config),
     Dir = filename:join([CaseDir, join(Rule, Opts)]),
     ok = filelib:ensure_dir(filename:join([Dir, dummy_file])),
     replace_path(CaseDir, Dir),
@@ -260,7 +266,7 @@ replace_path(PathA, PathB) ->
     true = code:add_patha(PathB).
 
 join(Rule, Opts) ->
-    string:join([atom_to_list(Rule)|lists:map(fun atom_to_list/1, Opts)], "_").
+    lists:join("_", [atom_to_list(Rule)|lists:map(fun atom_to_list/1, Opts)]).
 
 %%------------------------------------------------------------------------------
 %% Test cases
@@ -374,6 +380,12 @@ testExternal(Config, Rule, Opts) ->
     testSetOfTag:main(Rule),
     testSetTag:main(Rule).
 
+testExtensibilityImplied(Config) ->
+    test(Config, fun testExtensibilityImplied/3).
+testExtensibilityImplied(Config, Rule, Opts) ->
+    asn1_test_lib:compile("ExtensibilityImplied", Config,
+			  [Rule,no_ok_wrapper|Opts]),
+    testExtensibilityImplied:main().
 
 testChoPrim(Config) -> test(Config, fun testChoPrim/3).
 testChoPrim(Config, Rule, Opts) ->
@@ -433,6 +445,22 @@ testDEFAULT(Config, Rule, Opts) ->
     testDef:main(Rule),
     testSeqSetDefaultVal:main(Rule, Opts).
 
+testExtensionDefault(Config) ->
+    test(Config, fun testExtensionDefault/3).
+testExtensionDefault(Config, Rule, Opts) ->
+    asn1_test_lib:compile_all(["ExtensionDefault"], Config, [Rule|Opts]),
+    testExtensionDefault:main(Rule).
+
+testMaps(Config) ->
+    test(Config, fun testMaps/3,
+         [{ber,[maps,no_ok_wrapper]},
+          {ber,[maps,der,no_ok_wrapper]},
+          {per,[maps,no_ok_wrapper]},
+          {uper,[maps,no_ok_wrapper]}]).
+testMaps(Config, Rule, Opts) ->
+    asn1_test_lib:compile_all(['Maps'], Config, [Rule|Opts]),
+    testMaps:main(Rule).
+
 testOpt(Config) -> test(Config, fun testOpt/3).
 testOpt(Config, Rule, Opts) ->
     asn1_test_lib:compile("Opt", Config, [Rule|Opts]),
@@ -456,7 +484,7 @@ testSeqExtension(Config, Rule, Opts) ->
 			       "SeqExtension2"],
 			      Config,
                               [Rule|Opts]),
-    DataDir = ?config(data_dir, Config),
+    DataDir = proplists:get_value(data_dir, Config),
     testSeqExtension:main(Rule, DataDir, [Rule|Opts]).
 
 testSeqOptional(Config) -> test(Config, fun testSeqOptional/3).
@@ -561,41 +589,23 @@ testSetOfCho(Config, Rule, Opts) ->
     asn1_test_lib:compile("SetOfCho", Config, [Rule|Opts]),
     testSetOfCho:main(Rule).
 
-c_syntax(Config) ->
-    DataDir = ?config(data_dir, Config),
-    [{error, _} = asn1ct:compile(filename:join(DataDir, F))
-     || F <-["Syntax",
-             "BadTypeEnding",
-             "BadValueAssignment1",
-             "BadValueAssignment2",
-             "BadValueSet",
-             "ChoiceBadExtension",
-             "EnumerationBadExtension",
-             "Example",
-             "Export1",
-             "MissingEnd",
-             "SequenceBadComma",
-             "SequenceBadComponentName",
-             "SequenceBadComponentType",
-             "SeqBadComma"]].
-
 c_string(Config) ->
     test(Config, fun c_string/3).
 c_string(Config, Rule, Opts) ->
     asn1_test_lib:compile("String", Config, [Rule|Opts]),
     asn1ct:test('String').
 
-c_implicit_before_choice(Config) ->
-    test(Config, fun c_implicit_before_choice/3, [ber]).
-c_implicit_before_choice(Config, Rule, Opts) ->
-    DataDir = ?config(data_dir, Config),
-    CaseDir = ?config(case_dir, Config),
-    {error, _R2} = asn1ct:compile(filename:join(DataDir, "CCSNARG3"),
-                                  [Rule, {outdir, CaseDir}|Opts]).
-
 constraint_equivalence(Config) ->
-    DataDir = ?config(data_dir, Config),
-    CaseDir = ?config(case_dir, Config),
+    constraint_equivalence_abs(Config),
+    test(Config, fun constraint_equivalence/3).
+
+constraint_equivalence(Config, Rule, Opts) ->
+    M = 'ConstraintEquivalence',
+    asn1_test_lib:compile(M, Config, [Rule|Opts]).
+
+constraint_equivalence_abs(Config) ->
+    DataDir = proplists:get_value(data_dir, Config),
+    CaseDir = proplists:get_value(case_dir, Config),
     Asn1Spec = "ConstraintEquivalence",
     Asn1Src = filename:join(DataDir, Asn1Spec),
     ok = asn1ct:compile(Asn1Src, [abs,{outdir,CaseDir}]),
@@ -624,12 +634,12 @@ parse(Config) ->
     [asn1_test_lib:compile(M, Config, [abs]) || M <- test_modules()].
 
 per(Config) ->
-    test(Config, fun per/3, [per,uper]).
+    test(Config, fun per/3, [per,uper,{per,[maps]},{uper,[maps]}]).
 per(Config, Rule, Opts) ->
     [module_test(M, Config, Rule, Opts) || M <- per_modules()].
 
 ber_other(Config) ->
-    test(Config, fun ber_other/3, [ber]).
+    test(Config, fun ber_other/3, [ber,{ber,[maps]}]).
 
 ber_other(Config, Rule, Opts) ->
     [module_test(M, Config, Rule, Opts) || M <- ber_modules()].
@@ -637,12 +647,22 @@ ber_other(Config, Rule, Opts) ->
 der(Config) ->
     asn1_test_lib:compile_all(ber_modules(), Config, [der]).
 
-module_test(M, Config, Rule, Opts) ->
-    asn1_test_lib:compile(M, Config, [Rule|Opts]),
-    case asn1ct:test(list_to_atom(M), [{i, ?config(case_dir, Config)}]) of
-        ok    -> ok;
-        Error ->
-            erlang:error({test_failed, M, Opts, Error})
+module_test(M0, Config, Rule, Opts) ->
+    asn1_test_lib:compile(M0, Config, [Rule,?NO_MAPS_MODULE|Opts]),
+    case list_to_atom(M0) of
+	'LDAP' ->
+	    %% Because of the recursive definition of 'Filter' in
+	    %% the LDAP module, the construction of a sample
+	    %% value for 'Filter' is not guaranteed to terminate.
+	    ok;
+	M ->
+	    TestOpts = [{i, proplists:get_value(case_dir, Config)}],
+	    case asn1ct:test(M, TestOpts) of
+		ok ->
+		    ok;
+		Error ->
+		    erlang:error({test_failed, M, Opts, Error})
+	    end
     end.
 
 
@@ -765,6 +785,16 @@ testInfObjectClass(Config, Rule, Opts) ->
     testInfObjectClass:main(Rule),
     testInfObj:main(Rule).
 
+testUniqueObjectSets(Config) -> test(Config, fun testUniqueObjectSets/3).
+testUniqueObjectSets(Config, Rule, Opts) ->
+    CaseDir = proplists:get_value(case_dir, Config),
+    testUniqueObjectSets:main(CaseDir, Rule, Opts).
+
+testInfObjExtract(Config) -> test(Config, fun testInfObjExtract/3).
+testInfObjExtract(Config, Rule, Opts) ->
+    asn1_test_lib:compile("InfObjExtract", Config, [Rule|Opts]),
+    testInfObjExtract:main().
+
 testParam(Config) ->
     test(Config, fun testParam/3, [ber,{ber,[der]},per,uper]).
 testParam(Config, Rule, Opts) ->
@@ -804,18 +834,14 @@ testDeepTConstr(Config, Rule, Opts) ->
                               [Rule|Opts]),
     testDeepTConstr:main(Rule).
 
-testExport(Config) ->
-    {error, _} =
-	asn1ct:compile(filename:join(?config(data_dir, Config),
-				     "IllegalExport"),
-		       [{outdir, ?config(case_dir, Config)}]).
-
 testImport(Config) ->
     test(Config, fun testImport/3).
 testImport(Config, Rule, Opts) ->
-    Files = ["ImportsFrom","ImportsFrom2","ImportsFrom3"],
+    Files = ["ImportsFrom","ImportsFrom2","ImportsFrom3",
+	     "Importing","Exporting"],
     asn1_test_lib:compile_all(Files, Config, [Rule|Opts]),
     42 = 'ImportsFrom':i(),
+    testImporting:main(),
     ok.
 
 testMegaco(Config) -> test(Config, fun testMegaco/3).
@@ -839,24 +865,20 @@ testContextSwitchingTypes(Config, Rule, Opts) ->
 
 testTypeValueNotation(Config) -> test(Config, fun testTypeValueNotation/3).
 testTypeValueNotation(Config, Rule, Opts) ->
-    asn1_test_lib:compile_all(["SeqTypeRefPrim", "ValueTest"], Config,
-                              [Rule|Opts]),
+    asn1_test_lib:compile("SeqTypeRefPrim", Config, [Rule|Opts]),
     testTypeValueNotation:main(Rule, Opts).
 
+testValueTest(Config) -> test(Config, fun testValueTest/3).
+testValueTest(Config, Rule, Opts) ->
+    asn1_test_lib:compile("ValueTest", Config, [Rule|Opts]),
+    testValueTest:main().
+
 testOpenTypeImplicitTag(Config) ->
     test(Config, fun testOpenTypeImplicitTag/3).
 testOpenTypeImplicitTag(Config, Rule, Opts) ->
     asn1_test_lib:compile("OpenTypeImplicitTag", Config, [Rule|Opts]),
     testOpenTypeImplicitTag:main(Rule).
 
-duplicate_tags(Config) ->
-    DataDir = ?config(data_dir, Config),
-    CaseDir = ?config(case_dir, Config),
-    {error, [{error, {type, _, _, 'SeqOpt1Imp',
-			     {asn1, {duplicates_of_the_tags, _}}}}]} =
-	asn1ct:compile(filename:join(DataDir, "SeqOptional2"),
-		       [abs, {outdir, CaseDir}]).
-
 rtUI(Config) -> test(Config, fun rtUI/3).
 rtUI(Config, Rule, Opts) ->
     asn1_test_lib:compile("Prim", Config, [Rule|Opts]),
@@ -990,13 +1012,28 @@ testS1AP(Config, Rule, Opts) ->
 	    ok
     end.
 
+testRfcs() ->
+    [{timetrap,{minutes,90}}].
+
+testRfcs(Config) ->  test(Config, fun testRfcs/3,
+                          [{ber,[der,?NO_MAPS_MODULE]},
+                           {ber,[der,maps]}]).
+testRfcs(Config, Rule, Opts) ->
+    case erlang:system_info(system_architecture) of
+	"sparc-sun-solaris2.10" ->
+	    {skip,"Too slow for an old Sparc"};
+	_ ->
+	    testRfcs:compile(Config, Rule, Opts),
+	    testRfcs:test()
+    end.
+
 test_compile_options(Config) ->
     ok = test_compile_options:wrong_path(Config),
     ok = test_compile_options:path(Config),
     ok = test_compile_options:noobj(Config),
     ok = test_compile_options:record_name_prefix(Config),
     ok = test_compile_options:verbose(Config),
-    ok = test_compile_options:warnings_as_errors(Config).
+    ok = test_compile_options:maps(Config).
 
 testDoubleEllipses(Config) -> test(Config, fun testDoubleEllipses/3).
 testDoubleEllipses(Config, Rule, Opts) ->
@@ -1013,18 +1050,6 @@ test_modified_x420(Config, Rule, Opts) ->
     test_modified_x420:test(Config).
 
 
-testX420() ->
-    [{timetrap,{minutes,90}}].
-testX420(Config) ->
-    case erlang:system_info(system_architecture) of
-	"sparc-sun-solaris2.10" ->
-	    {skip,"Too slow for an old Sparc"};
-	_ ->
-	    Rule = ber,
-	    testX420:compile(Rule, [der], Config),
-	    ok = testX420:ticket7759(Rule, Config)
-    end.
-
 test_x691(Config) ->
     test(Config, fun test_x691/3, [per, uper]).
 test_x691(Config, Rule, Opts) ->
@@ -1055,7 +1080,7 @@ test_x691(Config, Rule, Opts) ->
     ok.
 
 ticket_6143(Config) ->
-    ok = test_compile_options:ticket_6143(Config).
+    asn1_test_lib:compile("AA1", Config, [?NO_MAPS_MODULE]).
 
 testExtensionAdditionGroup(Config) ->
     test(Config, fun testExtensionAdditionGroup/3).
@@ -1084,11 +1109,13 @@ test_modules() ->
      "CommonDataTypes",
      "Constraints",
      "ContextSwitchingTypes",
+     "CoverParser",
      "DS-EquipmentUser-CommonFunctionOrig-TransmissionPath",
      "Enum",
      "From",
      "H235-SECURITY-MESSAGES",
      "H323-MESSAGES",
+     "HighTagNumbers",
      "Import",
      "Int",
      "MAP-commonDataTypes",
@@ -1118,10 +1145,12 @@ test_modules() ->
      "Def",
      "Opt",
      "ELDAPv3",
-     "LDAP"].
+     "LDAP",
+     "SeqOptional2",
+     "CCSNARG3"].
 
 test_OTP_9688(Config) ->
-    PrivDir = ?config(case_dir, Config),
+    PrivDir = proplists:get_value(case_dir, Config),
     Data = "
 OTP-9688 DEFINITIONS ::= BEGIN
 
@@ -1140,44 +1169,58 @@ END
     ok = asn1ct:compile(File, [{outdir, PrivDir}]).
 
 
-timer_compile(Config, Rule) ->
-    asn1_test_lib:compile_all(["H235-SECURITY-MESSAGES", "H323-MESSAGES"],
-                              Config, [no_ok_wrapper,Rule]).
+timer_compile(Config, Opts0) ->
+    Files = ["H235-SECURITY-MESSAGES", "H323-MESSAGES"],
+    Opts = [no_ok_wrapper,?NO_MAPS_MODULE|Opts0],
+    asn1_test_lib:compile_all(Files, Config, Opts).
 
 testTimer_ber(Config) ->
-    timer_compile(Config, ber),
+    timer_compile(Config, [ber]),
     testTimer:go().
 
 testTimer_per(Config) ->
-    timer_compile(Config, per),
+    timer_compile(Config, [per]),
     testTimer:go().
 
 testTimer_uper(Config) ->
-    timer_compile(Config, uper),
+    timer_compile(Config, [uper]),
+    testTimer:go().
+
+testTimer_ber_maps(Config) ->
+    timer_compile(Config, [ber,maps]),
+    testTimer:go().
+
+testTimer_per_maps(Config) ->
+    timer_compile(Config, [per,maps]),
+    testTimer:go().
+
+testTimer_uper_maps(Config) ->
+    timer_compile(Config, [uper,maps]),
     testTimer:go().
 
 %% Test of multiple-line comment, OTP-8043
-testComment(suite) -> [];
 testComment(Config) ->
     asn1_test_lib:compile("Comment", Config, []),
     asn1_test_lib:roundtrip('Comment', 'Seq', {'Seq',12,true}).
 
-testName2Number(suite) -> [];
 testName2Number(Config) ->
-    N2NOptions = [{n2n,Type} || Type <- ['CauseMisc', 'CauseProtocol',
-                                         'CauseRadioNetwork',
-                                         'CauseTransport','CauseNas']],
-    asn1_test_lib:compile("S1AP-IEs", Config, N2NOptions),
+    N2NOptions0 = [{n2n,Type} ||
+                     Type <- ['Cause-Misc', 'CauseProtocol']],
+    N2NOptions = [?NO_MAPS_MODULE|N2NOptions0],
+    asn1_test_lib:compile("EnumN2N", Config, N2NOptions),
 
-    0 = 'S1AP-IEs':name2num_CauseMisc('control-processing-overload'),
-    'unknown-PLMN' = 'S1AP-IEs':num2name_CauseMisc(5),
+    0 = 'EnumN2N':'name2num_Cause-Misc'('control-processing-overload'),
+    'unknown-PLMN' = 'EnumN2N':'num2name_Cause-Misc'(5),
+    4 = 'EnumN2N':name2num_CauseProtocol('semantic-error'),
+    'transfer-syntax-error' = 'EnumN2N':num2name_CauseProtocol(0),
 
     %% OTP-10144
     %% Test that n2n option generates name2num and num2name functions supporting
     %% values not within the extension root if the enumeration type has an
     %% extension marker.
-    N2NOptionsExt = [{n2n, 'NoExt'}, {n2n, 'Ext'}, {n2n, 'Ext2'}],
+    N2NOptionsExt = [?NO_MAPS_MODULE,{n2n,'NoExt'},{n2n,'Ext'},{n2n,'Ext2'}],
     asn1_test_lib:compile("EnumN2N", Config, N2NOptionsExt),
+
     %% Previously, name2num and num2name was not generated if the type didn't
     %% have an extension marker:
     0 = 'EnumN2N':name2num_NoExt('blue'),
@@ -1195,9 +1238,11 @@ testName2Number(Config) ->
     ok.
 
 ticket_7407(Config) ->
-    asn1_test_lib:compile("EUTRA-extract-7407", Config, [uper]),
+    Opts = [uper,?NO_MAPS_MODULE],
+    asn1_test_lib:compile("EUTRA-extract-7407", Config, Opts),
     ticket_7407_code(true),
-    asn1_test_lib:compile("EUTRA-extract-7407", Config, [uper,no_final_padding]),
+    asn1_test_lib:compile("EUTRA-extract-7407", Config,
+                          [no_final_padding|Opts]),
     ticket_7407_code(false).
 
 ticket_7407_code(FinalPadding) ->
@@ -1272,16 +1317,72 @@ ticket7904(Config) ->
     {ok,_} = 'RANAPextract1':encode('InitiatingMessage', Val1),
     {ok,_} = 'RANAPextract1':encode('InitiatingMessage', Val1).
 
+
+%% Make sure that functions exported from other modules are
+%% actually used.
+
 xref(_Config) ->
-    xref:start(s),
-    xref:set_default(s, [{verbose,false},{warnings,false},{builtins,true}]),
+    S = ?FUNCTION_NAME,
+    xref:start(S),
+    xref:set_default(S, [{verbose,false},{warnings,false},{builtins,true}]),
     Test = filename:dirname(code:which(?MODULE)),
-    {ok,_PMs} = xref:add_directory(s, Test),
-    UnusedExports = "X - XU - asn1_appup_test - asn1_app_test - \".*_SUITE\" : Mod",
-    case xref:q(s, UnusedExports) of
+    {ok,_PMs} = xref:add_directory(S, Test),
+    Q = "X - XU - \".*_SUITE\" : Mod",
+    UnusedExports = xref:q(S, Q),
+    xref:stop(S),
+    case UnusedExports of
 	{ok,[]} ->
 	    ok;
 	{ok,[_|_]=Res} ->
 	    io:format("Exported, but unused: ~p\n", [Res]),
 	    ?t:fail()
     end.
+
+%% Ensure that all functions that are implicitly exported by
+%% 'export_all' in this module are actually used.
+
+xref_export_all(_Config) ->
+    S = ?FUNCTION_NAME,
+    xref:start(S),
+    xref:set_default(S, [{verbose,false},{warnings,false},{builtins,true}]),
+    {ok,_PMs} = xref:add_module(S, code:which(?MODULE)),
+    AllCalled = all_called(),
+    Def = "Called := " ++ lists:flatten(io_lib:format("~p", [AllCalled])),
+    {ok,_} = xref:q(S, Def),
+    {ok,Unused} = xref:q(S, "X - Called - range (closure E | Called)"),
+    xref:stop(S),
+    case Unused of
+        [] ->
+            ok;
+        [_|_] ->
+            Msg = [io_lib:format("~p:~p/~p\n", [M,F,A]) || {M,F,A} <- Unused],
+            io:format("There are unused functions:\n\n~s\n", [Msg]),
+            ?t:fail(unused_functions)
+    end.
+
+%% Collect all functions that common_test will call in this module.
+
+all_called() ->
+    [{?MODULE,end_per_group,2},
+     {?MODULE,end_per_suite,1},
+     {?MODULE,end_per_testcase,2},
+     {?MODULE,init_per_group,2},
+     {?MODULE,init_per_suite,1},
+     {?MODULE,init_per_testcase,2},
+     {?MODULE,suite,0}] ++
+        all_called_1(all() ++ groups()).
+
+all_called_1([{_,_}|T]) ->
+    all_called_1(T);
+all_called_1([{_Name,_Flags,Fs}|T]) ->
+    all_called_1(Fs ++ T);
+all_called_1([F|T]) when is_atom(F) ->
+    L = case erlang:function_exported(?MODULE, F, 0) of
+            false ->
+                [{?MODULE,F,1}];
+            true ->
+                [{?MODULE,F,0},{?MODULE,F,1}]
+        end,
+    L ++ all_called_1(T);
+all_called_1([]) ->
+    [].
