@@ -70,6 +70,9 @@ all_versions_tests() ->
      erlang_server_openssl_client,
      erlang_client_openssl_server_dsa_cert,
      erlang_server_openssl_client_dsa_cert,
+     erlang_client_openssl_server_anon,
+     erlang_server_openssl_client_anon,
+     erlang_server_openssl_client_anon_with_cert,
      erlang_server_openssl_client_reuse_session,
      erlang_client_openssl_server_renegotiate,
      erlang_client_openssl_server_nowrap_seqnum,
@@ -89,10 +92,13 @@ dtls_all_versions_tests() ->
      erlang_server_openssl_client,
      erlang_client_openssl_server_dsa_cert,
      erlang_server_openssl_client_dsa_cert,
+     erlang_client_openssl_server_anon,
+     erlang_server_openssl_client_anon,
+     erlang_server_openssl_client_anon_with_cert,
      erlang_server_openssl_client_reuse_session,
-     %%erlang_client_openssl_server_renegotiate,
-     %%erlang_client_openssl_server_nowrap_seqnum,
-     %%erlang_server_openssl_client_nowrap_seqnum,
+     erlang_client_openssl_server_renegotiate,
+     erlang_client_openssl_server_nowrap_seqnum,
+     erlang_server_openssl_client_nowrap_seqnum,
      erlang_client_openssl_server_no_server_ca_cert,
      erlang_client_openssl_server_client_cert,
      erlang_server_openssl_client_client_cert,
@@ -143,10 +149,15 @@ init_per_suite(Config0) ->
 	    try crypto:start() of
 		ok ->
                     ssl_test_lib:clean_start(),
-                  
-                    Config1 = ssl_test_lib:make_rsa_cert(Config0),
-                    Config2 = ssl_test_lib:make_dsa_cert(Config1),
-                    ssl_test_lib:cipher_restriction(Config2)
+                    Config = 
+                        case ssl_test_lib:openssl_dsa_support() of
+                            true ->
+                                Config1 = ssl_test_lib:make_rsa_cert(Config0),
+                                ssl_test_lib:make_dsa_cert(Config1);
+                            false ->
+                                ssl_test_lib:make_rsa_cert(Config0)
+                        end,
+                    ssl_test_lib:cipher_restriction(Config)
 		catch _:_  ->
 		    {skip, "Crypto did not start"}
 	    end
@@ -199,15 +210,27 @@ init_per_testcase(expired_session, Config) ->
     ssl:start(),
     Config;
 
-init_per_testcase(TestCase, Config) when TestCase == ciphers_rsa_signed_certs;
-					 TestCase == ciphers_dsa_signed_certs ->
-    ct:timetrap({seconds, 60}),
-    special_init(TestCase, Config);
-
+init_per_testcase(TestCase, Config) when 
+      TestCase == ciphers_dsa_signed_certs;
+      TestCase ==  erlang_client_openssl_server_dsa_cert;
+      TestCase == erlang_server_openssl_client_dsa_cert;
+      TestCase == erlang_client_openssl_server_dsa_cert;
+      TestCase ==  erlang_server_openssl_client_dsa_cert ->
+    case ssl_test_lib:openssl_dsa_support() of
+        true ->
+            special_init(TestCase, Config);
+        false ->
+           {skip, "DSA not supported by OpenSSL"}
+    end; 
 init_per_testcase(TestCase, Config) ->
-    ct:timetrap({seconds, 20}),
+    ct:timetrap({seconds, 35}),
     special_init(TestCase, Config).
 
+special_init(TestCase, Config) when 
+      TestCase == ciphers_rsa_signed_certs;
+      TestCase == ciphers_dsa_signed_certs->
+    ct:timetrap({seconds, 90}),
+    Config;
 special_init(TestCase, Config)
   when TestCase == erlang_client_openssl_server_renegotiate;
        TestCase == erlang_client_openssl_server_nowrap_seqnum;
@@ -533,7 +556,121 @@ erlang_server_openssl_client_dsa_cert(Config) when is_list(Config) ->
     ssl_test_lib:close_port(OpenSslPort),
     process_flag(trap_exit, false).
 
-%%-------------------------------------------------------------------- 
+%%--------------------------------------------------------------------
+erlang_client_openssl_server_anon() ->
+    [{doc,"Test erlang client with openssl server, anonymous"}].
+erlang_client_openssl_server_anon(Config) when is_list(Config) ->
+    process_flag(trap_exit, true),
+    %% OpenSSL expects a certificate and key, even if the cipher spec
+    %% is restructed to aNULL, so we use 'server_rsa_opts' here
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
+    ClientOpts = ssl_test_lib:ssl_options(client_anon_opts, Config),
+    VersionTuple = ssl_test_lib:protocol_version(Config, tuple),
+    Ciphers = ssl_test_lib:anonymous_suites(VersionTuple),
+
+    {ClientNode, _, Hostname} = ssl_test_lib:run_where(Config),
+
+    Data = "From openssl to erlang",
+
+    Port = ssl_test_lib:inet_port(node()),
+    CertFile = proplists:get_value(certfile, ServerOpts),
+    KeyFile = proplists:get_value(keyfile, ServerOpts),
+    Version = ssl_test_lib:protocol_version(Config),
+    Exe = "openssl",
+    Args = ["s_server", "-accept", integer_to_list(Port),
+	    ssl_test_lib:version_flag(Version),
+            "-cert", CertFile, "-key", KeyFile,
+	    "-cipher", "aNULL", "-msg"],
+
+    OpensslPort =  ssl_test_lib:portable_open_port(Exe, Args),
+
+    ssl_test_lib:wait_for_openssl_server(Port, proplists:get_value(protocol, Config)),
+
+    Client = ssl_test_lib:start_client([{node, ClientNode}, {port, Port},
+					{host, Hostname},
+					{from, self()},
+					{mfa, {?MODULE,
+					       erlang_ssl_receive, [Data]}},
+					{options, [{ciphers, Ciphers} | ClientOpts]}]),
+
+    true = port_command(OpensslPort, Data),
+
+    ssl_test_lib:check_result(Client, ok),
+
+    %% Clean close down!   Server needs to be closed first !!
+    ssl_test_lib:close_port(OpensslPort),
+    ssl_test_lib:close(Client),
+    process_flag(trap_exit, false),
+    ok.
+%%--------------------------------------------------------------------
+erlang_server_openssl_client_anon() ->
+    [{doc,"Test erlang server with openssl client, anonymous"}].
+erlang_server_openssl_client_anon(Config) when is_list(Config) ->
+    process_flag(trap_exit, true),
+    ServerOpts = ssl_test_lib:ssl_options(server_anon_opts, Config),
+    VersionTuple = ssl_test_lib:protocol_version(Config, tuple),
+    Ciphers = ssl_test_lib:anonymous_suites(VersionTuple),
+
+    {_, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+
+    Data = "From openssl to erlang",
+
+    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+                                       {from, self()},
+                                       {mfa, {?MODULE, erlang_ssl_receive, [Data]}},
+                                       {options, [{ciphers, Ciphers} | ServerOpts]}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Version = ssl_test_lib:protocol_version(Config),
+    Exe = "openssl",
+    Args = ["s_client", "-connect", hostname_format(Hostname) ++ ":" ++ integer_to_list(Port),
+	    ssl_test_lib:version_flag(Version),
+	    "-cipher", "aNULL", "-msg"],
+
+    OpenSslPort = ssl_test_lib:portable_open_port(Exe, Args),
+    true = port_command(OpenSslPort, Data),
+
+    ssl_test_lib:check_result(Server, ok),
+
+    %% Clean close down!   Server needs to be closed first !!
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close_port(OpenSslPort),
+    process_flag(trap_exit, false).
+
+  %%--------------------------------------------------------------------
+  erlang_server_openssl_client_anon_with_cert() ->
+    [{doc,"Test erlang server with openssl client, anonymous (with cert)"}].
+  erlang_server_openssl_client_anon_with_cert(Config) when is_list(Config) ->
+    process_flag(trap_exit, true),
+    ServerOpts = ssl_test_lib:ssl_options(server_rsa_opts, Config),
+    VersionTuple = ssl_test_lib:protocol_version(Config, tuple),
+    Ciphers = ssl_test_lib:anonymous_suites(VersionTuple),
+
+    {_, ServerNode, Hostname} = ssl_test_lib:run_where(Config),
+
+    Data = "From openssl to erlang",
+
+    Server = ssl_test_lib:start_server([{node, ServerNode}, {port, 0},
+                                       {from, self()},
+                                       {mfa, {?MODULE, erlang_ssl_receive, [Data]}},
+                                       {options, [{ciphers, Ciphers} | ServerOpts]}]),
+    Port = ssl_test_lib:inet_port(Server),
+    Version = ssl_test_lib:protocol_version(Config),
+    Exe = "openssl",
+    Args = ["s_client", "-connect", hostname_format(Hostname) ++ ":" ++ integer_to_list(Port),
+  	    ssl_test_lib:version_flag(Version),
+  	    "-cipher", "aNULL", "-msg"],
+
+    OpenSslPort = ssl_test_lib:portable_open_port(Exe, Args),
+    true = port_command(OpenSslPort, Data),
+
+    ssl_test_lib:check_result(Server, ok),
+
+    %% Clean close down!   Server needs to be closed first !!
+    ssl_test_lib:close(Server),
+    ssl_test_lib:close_port(OpenSslPort),
+    process_flag(trap_exit, false).
+
+%%--------------------------------------------------------------------
 
 erlang_server_openssl_client_reuse_session() ->
     [{doc, "Test erlang server with openssl client that reconnects with the"
@@ -1016,7 +1153,7 @@ ssl2_erlang_server_openssl_client(Config) when is_list(Config) ->
     
     ct:log("Ports ~p~n", [[erlang:port_info(P) || P <- erlang:ports()]]), 
     consume_port_exit(OpenSslPort),
-    ssl_test_lib:check_result(Server, {error, {tls_alert, "handshake failure"}}),
+    ssl_test_lib:check_result(Server, {error, {tls_alert, "bad record mac"}}),
     process_flag(trap_exit, false).
 %%--------------------------------------------------------------------
 ssl2_erlang_server_openssl_client_comp() ->
